@@ -692,15 +692,14 @@ static void _archive_remove (FRArchive *archive, GList *file_list);
 
 
 static GList *
-escape_file_list (GList *file_list) {
+escape_file_list (GList *file_list) 
+{
 	GList *e_file_list = NULL;
 	GList *scan;
 
 	for (scan = file_list; scan; scan = scan->next) {
-		gchar *filename = scan->data;
-
-		e_file_list = g_list_prepend (e_file_list, 
-					      shell_escape (filename));
+		char *filename = scan->data;
+		e_file_list = g_list_prepend (e_file_list, shell_escape (filename));
 	}
 
 	return e_file_list;
@@ -874,8 +873,8 @@ file_list_remove_from_pattern (GList      **list,
 		char *path = scan->data;
 		char *utf8_name;
 
-		utf8_name = g_locale_to_utf8 (file_name_from_path (path), 
-					      -1, NULL, NULL, NULL);
+		utf8_name = g_filename_to_utf8 (file_name_from_path (path), 
+						-1, NULL, NULL, NULL);
 		if (match_patterns (patterns, utf8_name, FNM_CASEFOLD)) {
 			*list = g_list_remove_link (*list, scan);
 			g_free (scan->data);
@@ -1061,13 +1060,19 @@ static void
 _archive_remove (FRArchive *archive,
 		 GList     *file_list)
 {
-	GList *e_file_list;
-	GList *scan;
+	gboolean  file_list_created = FALSE;
+	GList    *e_file_list;
+	GList    *scan;
 
 	/* file_list == NULL means delete all files in archive. */
-
-	if (file_list == NULL) 
-		file_list = archive->command->file_list;
+	
+	if (file_list == NULL) {
+		for (scan = archive->command->file_list; scan != NULL; scan = scan->next) {
+			FileData *fdata = (FileData*) scan->data;
+			file_list = g_list_prepend (file_list, fdata->original_path);
+		}
+		file_list_created = TRUE;
+	} 
 
 	e_file_list = escape_file_list (file_list);
 	fr_command_set_n_files (archive->command, g_list_length (e_file_list));
@@ -1093,6 +1098,8 @@ _archive_remove (FRArchive *archive,
 		prev->next = scan;
 	}
 
+	if (file_list_created)
+		g_list_free (file_list);
 	path_list_free (e_file_list);
 }
 
